@@ -1157,6 +1157,44 @@ void doMkdir(params)
 
 
 
+/****************************************************************************
+** 	_doMerge
+**
+**	Purpose	: Merge elements of an array into a string
+**	Args	: text string and separating character
+**	Returns	: 
+**	Notes	: 
+*/
+
+void doMerge(params)
+	plist_t	*params;
+{
+	char	*tok;
+	sym_t	*array,
+		*cur;
+	int	num,
+		count;
+	char	buf[2048];
+
+	array = params->sym;
+	tok = params->next->sym->val;
+	num = symGetNumArrayElements(array);
+	count = 0;
+	*buf = 0;
+	while(count < num)
+	{
+		cur = symGetArrayElement(array,count);
+		if (count > 0)
+			strcat(buf,tok);
+		if (cur->val)
+			strcat(buf, cur->val);
+		count++;
+	}
+	externReturn = createCharSymbol(buf);
+}
+
+
+
 /****************************************************************************
 ** 	_doSplit
 **
