@@ -957,9 +957,12 @@ sym_t *createCharSymbol(val)
 	clearSymbol(new);
 	*(new->name) = 0;
 	new->type = TYPE_CHAR;
-	new->length = strlen(val) + 1;
 	new->array = SCALAR;
-	new->val = (char *)strdup(val);
+	if (val)
+		new->val = (char *)strdup(val);
+	else
+		new->val = (char *)strdup("");
+	new->length = strlen(new->val) + 1;
 	return(new);
 }
 
