@@ -11,7 +11,7 @@ do
 	echo "	@echo					;\\"
 	echo "	echo \"--> [$DIR] directory  \"        	;\\"
 	echo "	cd $DIR                            	;\\"
-	echo "	make \$(MFLAGS)	$targ			;\\"
+	echo "	\$(MAKE) \$(MFLAGS)	$targ			;\\"
 	echo "	echo \"<-- [$DIR] done       \"		"
 	echo ""
 done
