@@ -1,6 +1,6 @@
 #!/bin/sh
 
-. $TOP/makegen/makegen.cf
+. $MACRO_DIR/makegen/makegen.cf
 
 obj=$1
 src=$2
@@ -18,6 +18,3 @@ echo
 echo	"clean ::"
 echo	"	rm -f $obj $base.lint"
 echo
-echo	"depend ::"
-echo	'	@$(TOP)/makedepend/makedepend -a -fMakefile.full $(CC_FLAGS)' $src 2>/dev/null
-echo
