@@ -36,6 +36,7 @@
 #endif
 
 
+#include <common/site.h>
 #include <common/portability.h>
 #include "msql_priv.h"
 
@@ -109,7 +110,7 @@ int msqlLoadAcl(verbose)
 	acc_t	*accNew,
 		*accTail;
 	char	buf[1024],
-		path[255],
+		path[MAXPATHLEN],
 		*tok;
 	FILE	*fp;
 	int	newEntry,
@@ -119,7 +120,7 @@ int msqlLoadAcl(verbose)
 	/*
 	** Open the acl file
 	*/
-	(void)sprintf(path,"%s/msql.acl", 
+	(void)snprintf(path,MAXPATHLEN,"%s/msql.acl", 
 		(char *)msqlGetCharConf("general", "inst_dir"));
 	fp = fopen(path,"r");
 	if (!fp)
@@ -129,7 +130,8 @@ int msqlLoadAcl(verbose)
 			fprintf(stderr,
 			   "\nWarning : No ACL file.  Using global read/write access.\n");
 		}
-		sprintf(packet,"-1:Warning - Couldn't open ACL file\n");
+		snprintf(packet,PKT_LEN, 
+			"-1:Warning - Couldn't open ACL file\n");
 		return(-1);
 	}
 
@@ -170,7 +172,7 @@ int msqlLoadAcl(verbose)
 			{
 				ERR(("Bad entry header location at line %d\n",
 					lineNum));
-				sprintf(packet,
+				snprintf(packet,PKT_LEN,
 				   "-1:Bad entry header location at line %d\n"
 					,lineNum);
 				fclose(fp);
@@ -208,7 +210,7 @@ int msqlLoadAcl(verbose)
 			{
 				ERR(("Bad entry header location at line %d\n",
 					lineNum));
-				sprintf(packet, 
+				snprintf(packet, PKT_LEN,
 				    "-1:Bad entry header location at line %d\n"
 					,lineNum);
 				fclose(fp);
@@ -258,7 +260,7 @@ int msqlLoadAcl(verbose)
 			{
 				ERR(("Bad entry header location at line %d\n",
 					lineNum));
-				sprintf(packet,
+				snprintf(packet,PKT_LEN,
 				    "-1:Bad entry header location at line %d\n"
 					,lineNum);
 				fclose(fp);
@@ -308,7 +310,7 @@ int msqlLoadAcl(verbose)
 			{
 				ERR(("Bad entry header location at line %d\n",
 					lineNum));
-				sprintf(packet,
+				snprintf(packet,PKT_LEN,
 				    "-1:Bad entry header location at line %d\n"
 					,lineNum);
 				fclose(fp);
@@ -358,7 +360,7 @@ int msqlLoadAcl(verbose)
 			{
 				ERR(("Bad entry header location at line %d\n",
 					lineNum));
-				sprintf(packet,
+				snprintf(packet,PKT_LEN,
 				    "-1:Bad entry header location at line %d\n"
 					,lineNum);
 				fclose(fp);
@@ -399,7 +401,7 @@ int msqlLoadAcl(verbose)
 			{
 				ERR(("Bad entry header location at line %d\n",
 					lineNum));
-				sprintf(packet,
+				snprintf(packet,PKT_LEN,
 				    "-1:Bad entry header location at line %d\n"
 					,lineNum);
 				fclose(fp);
@@ -438,7 +440,7 @@ int msqlLoadAcl(verbose)
 		    default:
 			ERR(("Unknown ACL command \"%s\" at line %d\n", 
 				tok,lineNum));
-			sprintf(packet,
+			snprintf(packet,PKT_LEN,
 				"-1:Unknown ACL command \"%s\" at line %d\n", 
 				tok,lineNum);
 			fclose(fp);
