@@ -828,6 +828,25 @@ avl_nod *avlGetFirst(tree)
 }
 
 
+avl_nod *avlGetLast(tree)
+	avltree	*tree;
+{
+	avl_nod	*cur;
+
+	cur = avlGetNode(tree,tree->sblk->root);
+	tree->curNode = 0;
+	if (!cur)
+		return(NULL);
+	while(cur->right)
+	{
+		cur = avlGetNode(tree,cur->right);
+	}
+	tree->curNode = cur->nodeNum;
+	return(cur);
+}
+
+
+
 avl_nod *avlGetNext(tree,cursor)
 	avltree	*tree;
 	avl_cur	*cursor;
