@@ -399,17 +399,20 @@ static int compareIndexValues(val1,val2)
 **      Notes   : 
 */
 
-int updateIndices(entry,oldFields,pos,row,flist)
+int updateIndices(entry,oldFields,pos,row,flist, updated, candIdx)
 	cache_t	*entry;
 	field_t	*oldFields;
 	u_int	pos;
         row_t 	*row;
-	int	*flist;
+	int	*flist,
+		*updated,
+		candIdx;
 {
 	mindex_t	*curIndex;
 	int		res,
 			count,
 			offset,
+			idxCount,
 			needUpdate;
 	field_t		*newFields = NULL,
 			*newTail = NULL,
@@ -444,6 +447,8 @@ int updateIndices(entry,oldFields,pos,row,flist)
 	** Work through the indices
 	*/
 
+	idxCount = 0;
+	*updated = 0;
 	curIndex = entry->indices;
 	while(curIndex)
 	{
@@ -478,6 +483,8 @@ int updateIndices(entry,oldFields,pos,row,flist)
 		}
 
 
+		if (idxCount == candIdx)
+			*updated = 1;
 		if (fillIndexBuffer(curIndex,oldFields) < 0)
 			return(-1);
 		if (avlDelete(curIndex->tree, curIndex->buf, (off_t)pos) < 0)
@@ -488,6 +495,7 @@ int updateIndices(entry,oldFields,pos,row,flist)
 			return(-1);
 
 		curIndex = curIndex->next;
+		idxCount++;
 	}
 
 	/*
@@ -1022,6 +1030,11 @@ cand_t *msqlSetupCandidate(entry, conds, fields, ignoreIdent)
 			curCond = conds;
 			while(curCond)
 			{
+				if (curCond->subCond)
+				{
+					curCond = curCond->next;
+					continue;
+				}
 				if(curCond->fieldID==candIndex->fields[count])
 				{
 					if (curCond->value->nullVal)
