@@ -293,7 +293,7 @@ void msqlClose(sock)
 	int	sock;
 {
 	setServerSock(sock);
-	sprintf(packet,"%d:\n",QUIT);
+	snprintf(packet,PKT_LEN,"%d:\n",QUIT);
 	writePkt(sock);
 	closeServer(sock);
 }
@@ -398,7 +398,7 @@ static m_fdata *tableToFieldList(data)
 {
 	m_data	*curRow;
 	m_fdata	*curField,
-		*prevField,
+		*prevField = NULL,
 		*head = NULL;
 
 	curRow = data;
@@ -509,7 +509,7 @@ int msqlConnect(host)
 		sock = socket(AF_UNIX,SOCK_STREAM,0);
 		if (sock < 0)
 		{
-			sprintf(msqlErrMsg,SOCK_ERROR);
+			snprintf(msqlErrMsg,MAX_ERR_MSG,SOCK_ERROR);
 			return(-1);
 		}
 		setServerSock(sock);
@@ -523,7 +523,7 @@ int msqlConnect(host)
 		if(connect(sock,(struct sockaddr *) &UNIXaddr, 
 			sizeof(UNIXaddr))<0)
 		{
-			sprintf(msqlErrMsg,CONNECTION_ERROR);
+			snprintf(msqlErrMsg,MAX_ERR_MSG,CONNECTION_ERROR);
 #ifdef OS2
 			soclose(sock);
 #else
@@ -536,13 +536,13 @@ int msqlConnect(host)
 	else
 	{
 	        tcpPort = msqlGetIntConf("general","tcp_port");
-		sprintf(hostInfo,"%s via TCP/IP",host);
+		snprintf(hostInfo,sizeof(hostInfo),"%s via TCP/IP",host);
 		msqlDebug(MOD_API,"Server name = %s.  Using TCP sock (%d)\n",
 			host, tcpPort);
 		sock = socket(AF_INET,SOCK_STREAM,0);
 		if (sock < 0)
 		{
-			sprintf(msqlErrMsg,IPSOCK_ERROR);
+			snprintf(msqlErrMsg,MAX_ERR_MSG,IPSOCK_ERROR);
 			return(-1);
 		}
 		setServerSock(sock);
@@ -566,7 +566,7 @@ int msqlConnect(host)
 			hp = gethostbyname(host);
 			if (!hp)
 			{
-				sprintf(msqlErrMsg,
+				snprintf(msqlErrMsg,MAX_ERR_MSG,
 					UNKNOWN_HOST,
 					host);
 #ifdef OS2
@@ -582,7 +582,7 @@ int msqlConnect(host)
 		if(connect(sock,(struct sockaddr *) &IPaddr, 
 			sizeof(IPaddr))<0)
 		{
-			sprintf(msqlErrMsg,CONN_HOST_ERROR, host);
+			snprintf(msqlErrMsg,MAX_ERR_MSG,CONN_HOST_ERROR,host);
 #ifdef OS2
 			soclose(sock);
 #else
@@ -643,7 +643,7 @@ int msqlConnect(host)
 	version = atoi(cp + 1);
 	if (version != PROTOCOL_VERSION) 
 	{
-		sprintf(msqlErrMsg, VERSION_ERROR, version, 
+		snprintf(msqlErrMsg, MAX_ERR_MSG, VERSION_ERROR, version, 
 			PROTOCOL_VERSION);
 		closeServer(sock);
 		return(-1);
@@ -677,11 +677,11 @@ int msqlConnect(host)
 		psz = getenv("USER");
 		if (!psz)
 			psz = "unknown";
-		(void)sprintf( packet, "%s\n", psz );
+		(void)snprintf(packet,PKT_LEN, "%s\n", psz );
 	}
 	else
 	{
-		(void)sprintf( packet,"x_%s\n", user );
+		(void)snprintf(packet,PKT_LEN,"x_%s\n", user );
 	}
 #else
 		
@@ -697,10 +697,10 @@ int msqlConnect(host)
 			closeServer(sock);
 			return(-1);
 		}
-		(void)sprintf(packet,"%s\n",psz);
+		(void)snprintf(packet,PKT_LEN,"%s\n",psz);
 	}
 	else
-		(void)sprintf(packet,"%s\n",pw->pw_name);
+		(void)snprintf(packet,PKT_LEN,"%s\n",pw->pw_name);
 #endif
 	writePkt(sock);
 	if(readPkt(sock) <= 0)
@@ -763,7 +763,7 @@ int msqlSelectDB(sock,db)
 	** Issue the init DB command
 	*/
 
-	(void)sprintf(packet,"%d:%s\n",INIT_DB,db);
+	(void)snprintf(packet,PKT_LEN,"%d:%s\n",INIT_DB,db);
 	writePkt(sock);
 	if(readPkt(sock) <= 0)
 	{
@@ -971,7 +971,7 @@ int readQueryData(sock)
 		numRows;
 	char	*cp,
 		*tmpcp;
-	m_data	*cur;
+	m_data	*cur = NULL;
 	
 	if (readPkt(sock) <= 0)
 	{
@@ -1078,7 +1078,7 @@ int msqlQuery(sock,q)
 	** Issue the query
 	*/
 
-	(void)sprintf(packet,"%d:%s\n",QUERY,q);
+	(void)snprintf(packet,PKT_LEN,"%d:%s\n",QUERY,q);
 	writePkt(sock);
 	if(readPkt(sock) <= 0)
 	{
@@ -1131,8 +1131,11 @@ int msqlQuery(sock,q)
 	if (queryData)
 	{
 		freeQueryData(queryData);
-		freeQueryData(fieldData);
 		queryData = NULL;
+	}
+	if (fieldData)
+	{
+		freeQueryData(fieldData);
 		fieldData = NULL;
 	}
 
@@ -1182,7 +1185,7 @@ m_result *msqlListDBs(sock)
 	(void)bzero(tmp, sizeof(m_result));
 	msqlDebug(MOD_MALLOC,"Result Handle - malloc @ %X of %d\n",
 		tmp, sizeof(m_result));
-	sprintf(packet,"%d:\n",DB_LIST);
+	snprintf(packet,PKT_LEN,"%d:\n",DB_LIST);
 	writePkt(sock);
 	numFields = 1;
 	tmp->numRows = readQueryData(sock);
@@ -1235,7 +1238,7 @@ m_result *msqlListTables(sock)
 	msqlDebug(MOD_MALLOC,"Result Handle - malloc @ %X of %d\n",
 		tmp, sizeof(m_result));
 	(void)bzero(tmp, sizeof(m_result));
-	sprintf(packet,"%d:\n",TABLE_LIST);
+	snprintf(packet,PKT_LEN,"%d:\n",TABLE_LIST);
 	writePkt(sock);
 	numFields = 1;
 	tmp->numRows = readQueryData(sock);
@@ -1288,7 +1291,7 @@ m_result *msqlListFields(sock,table)
 	msqlDebug(MOD_MALLOC,"Result Handle - malloc @ %X of %d\n",
 		tmp, sizeof(m_result));
 	(void)bzero(tmp, sizeof(m_result));
-	sprintf(packet,"%d:%s\n",FIELD_LIST,table);
+	snprintf(packet,PKT_LEN,"%d:%s\n",FIELD_LIST,table);
 	writePkt(sock);
 	numFields = 6;
 	tmp->numFields = readQueryData(sock);
@@ -1325,7 +1328,7 @@ m_seq *msqlGetSequenceInfo(sock,table)
 	char	*cp;
 
 	msqlDebug(MOD_API,"msqlGetSequenceInfo(%d,%s)\n",sock,table);
-	sprintf(packet,"%d:%s\n",SEQ_INFO,table);
+	snprintf(packet,PKT_LEN,"%d:%s\n",SEQ_INFO,table);
 	writePkt(sock);
 
         if(readPkt(sock) <= 0)
@@ -1388,7 +1391,7 @@ m_result *msqlListIndex(sock,table, index)
 	msqlDebug(MOD_MALLOC,"Result Handle - malloc @ %X of %d\n",
 		tmp, sizeof(m_result));
 	(void)bzero(tmp, sizeof(m_result));
-	sprintf(packet,"%d:%s:%s\n",INDEX_LIST,table,index);
+	snprintf(packet,PKT_LEN,"%d:%s:%s\n",INDEX_LIST,table,index);
 	writePkt(sock);
 	numFields = 1;
 	tmp->numFields = readQueryData(sock);
@@ -1425,7 +1428,7 @@ int msqlCreateDB(sock,db)
 	char	*cp;
 
 	msqlDebug(MOD_API,"msqlCreateDB(%d,%s)\n",sock,db);
-	sprintf(packet,"%d:%s\n",CREATE_DB,db);
+	snprintf(packet,PKT_LEN,"%d:%s\n",CREATE_DB,db);
 	writePkt(sock);
 	if(readPkt(sock) <= 0)
 	{
@@ -1466,7 +1469,7 @@ int msqlCopyDB(sock,fromDB, toDB)
 	char	*cp;
 
 	msqlDebug(MOD_API,"msqlCopyDB(%d,%s,%s)\n",sock,fromDB, toDB);
-	sprintf(packet,"%d:%s:%s\n",COPY_DB,fromDB, toDB);
+	snprintf(packet,PKT_LEN,"%d:%s:%s\n",COPY_DB,fromDB, toDB);
 	writePkt(sock);
 	if(readPkt(sock) <= 0)
 	{
@@ -1507,7 +1510,7 @@ int msqlMoveDB(sock,fromDB, toDB)
 	char	*cp;
 
 	msqlDebug(MOD_API,"msqlMoveDB(%d,%s,%s)\n",sock,fromDB, toDB);
-	sprintf(packet,"%d:%s:%s\n",MOVE_DB,fromDB, toDB);
+	snprintf(packet,PKT_LEN,"%d:%s:%s\n",MOVE_DB,fromDB, toDB);
 	writePkt(sock);
 	if(readPkt(sock) <= 0)
 	{
@@ -1546,7 +1549,7 @@ int msqlDropDB(sock,db)
 	char	*cp;
 
 	msqlDebug(MOD_API,"msqlDropDB(%d,%s)\n",sock,db);
-	sprintf(packet,"%d:%s\n",DROP_DB,db);
+	snprintf(packet,PKT_LEN,"%d:%s\n",DROP_DB,db);
 	writePkt(sock);
 	if(readPkt(sock) <= 0)
 	{
@@ -1584,7 +1587,7 @@ int msqlShutdown(sock)
 	char	*cp;
 
 	msqlDebug(MOD_API,"msqlShutdown(%d)\n",sock);
-	sprintf(packet,"%d:\n",SHUTDOWN);
+	snprintf(packet,PKT_LEN,"%d:\n",SHUTDOWN);
 	writePkt(sock);
 	if(readPkt(sock) <= 0)
 	{
@@ -1623,7 +1626,7 @@ int msqlReloadAcls(sock)
 	char	*cp;
 
 	msqlDebug(MOD_API,"msqlReloadAcl(%d)\n",sock);
-	sprintf(packet,"%d:\n",RELOAD_ACL);
+	snprintf(packet,PKT_LEN,"%d:\n",RELOAD_ACL);
 	writePkt(sock);
 	if(readPkt(sock) <= 0)
 	{
@@ -1678,7 +1681,7 @@ int msqlGetServerStats(sock)
 	int	sock;
 {
 	msqlDebug(MOD_API,"msqlShutdown(%d)\n",sock);
-	sprintf(packet,"%d:\n",SERVER_STATS);
+	snprintf(packet,PKT_LEN,"%d:\n",SERVER_STATS);
 	writePkt(sock);
 	if(readPkt(sock) <= 0)
 	{
