@@ -3,7 +3,7 @@
 **
 **
 ** Copyright (c) 1993-95  David J. Hughes
-** Copyright (c) 1995   Hughes Technologies Pty Ltd
+** Copyright (c) 1995-97  Hughes Technologies Pty Ltd
 **
 ** Permission to use, copy, and distribute for non-commercial purposes,
 ** is hereby granted without fee, providing that the above copyright
@@ -12,21 +12,23 @@
 **
 ** This software is provided "as is" without any expressed or implied warranty.
 **
-** ID = "$Id:"
-**
 */
 
 
-#ifndef lint
-static char RCS_id[] = 
-	"msql.c,v 1.3 1994/08/19 08:03:09 bambi Exp";
-#endif 
-
-
 #include <stdio.h>
 #include <sys/types.h>
 #include <fcntl.h>
-#include <paths.h>
+#ifdef OS2
+#  include <common/getopt.h>
+#else
+#  include <stdlib.h>
+#  include <unistd.h>
+#  include <string.h>
+#endif
+
+#ifdef WIN32
+#  include <winsock.h>
+#endif
 
 #include "common/portability.h"
 
@@ -34,17 +36,20 @@ static char RCS_id[] =
 #include "version.h"
 
 
-#define	MAX_LEN 15 * 1024
+#define	MAX_LEN 25 * 1024
 
 
-usage()
+char *msql_tmpnam();
+
+
+void usage()
 {
-	(void)fprintf(stderr,"\n\nUsage : msql [-h host] database\n\n");
+	(void)fprintf(stderr,"\n\nUsage : msql [-f conf_file] [-h host] database\n\n");
 }
 
 
 
-help()
+void help()
 {
 	(void)printf("\n\nMiniSQL Help!\n\n");
 	(void)printf("The following commands are available :- \n\n");
@@ -55,6 +60,10 @@ help()
 }
 
 
+#ifdef max
+#  undef max
+#endif
+
 int max(v1,v2)
 	int	v1,
 		v2;
@@ -66,7 +75,7 @@ int max(v1,v2)
 }
 
 
-bufFill(buf,length,max,filler)
+void bufFill(buf,length,max,filler)
 	char	*buf;
 	int	length,
 		max;
@@ -86,7 +95,7 @@ bufFill(buf,length,max,filler)
 
 
 
-fill(length,max,filler)
+void fill(length,max,filler)
 	int	length,
 		max;
 	char	filler;
@@ -102,15 +111,15 @@ fill(length,max,filler)
 
 
 
-handleQuery(sock, q)
+void handleQuery(sock, q)
 	int	sock;
 	char	*q;
 {
-	char	*nq,
-		sepBuf[MAX_LEN];
+	char	sepBuf[MAX_LEN];
 	int	off,
 		maxLen,
-		length;
+		length,
+		res;
 	m_result *result;
 	m_row	cur;
 	m_field	*curField;
@@ -126,12 +135,17 @@ handleQuery(sock, q)
 		return;
 	}
 
-	if (msqlQuery(sock,q) < 0)
+	res = msqlQuery(sock,q);
+	if (res < 0)
 	{
+#ifdef OS2
+		printf("\n\nERROR : %s\n\n",msqlGetErrMsg(msqlErrMsg));
+#else
 		printf("\n\nERROR : %s\n\n",msqlErrMsg);
+#endif
 		return;
 	}
-	printf("\nQuery OK.\n\n");
+	printf("\nQuery OK.  %d row(s) modified or retrieved.\n\n",res);
 
 	result = msqlStoreResult();
 	if (!result)
@@ -140,8 +154,6 @@ handleQuery(sock, q)
 		return;
 	}
 
-	printf("%d rows matched.\n\n",msqlNumRows(result));
-
 	/*
 	** Print a pretty header .... 
 	*/
@@ -161,6 +173,9 @@ handleQuery(sock, q)
 				break;
 
 		    	case INT_TYPE:
+		    	case UINT_TYPE:
+		    	case MONEY_TYPE:
+		    	case TIME_TYPE:
 				length = strlen(curField->name);
 				if (length < 8)
 				{
@@ -168,7 +183,16 @@ handleQuery(sock, q)
 				}
 				break;
 
+		    	case DATE_TYPE:
+				length = strlen(curField->name);
+				if (length < 11)
+				{
+					length = 11;
+				}
+				break;
+
 		    	case CHAR_TYPE:
+		    	case TEXT_TYPE:
 				length = max(strlen(curField->name),
 						curField->length);
 				break;
@@ -193,6 +217,9 @@ handleQuery(sock, q)
 		switch(curField->type)
 		{
 		    	case INT_TYPE:
+		    	case UINT_TYPE:
+		    	case MONEY_TYPE:
+		    	case TIME_TYPE:
 				length = strlen(curField->name);
 				if (length < 8)
 				{
@@ -200,6 +227,14 @@ handleQuery(sock, q)
 				}
 				break;
 
+		    	case DATE_TYPE:
+				length = strlen(curField->name);
+				if (length < 11)
+				{
+					length = 11;
+				}
+				break;
+
 		    	case REAL_TYPE:
 				length = strlen(curField->name);
 				if (length < 12)
@@ -209,6 +244,7 @@ handleQuery(sock, q)
 				break;
 
 		    	case CHAR_TYPE:
+		    	case TEXT_TYPE:
 				length = max(strlen(curField->name),
 						curField->length);
 				break;
@@ -236,6 +272,9 @@ handleQuery(sock, q)
 			switch(curField->type)
 			{
 			    case INT_TYPE:
+			    case UINT_TYPE:
+			    case MONEY_TYPE:
+			    case TIME_TYPE:
 				length = strlen(curField->name);
 				if (length < 8)
 				{
@@ -243,6 +282,14 @@ handleQuery(sock, q)
 				}
 				break;
 
+			    case DATE_TYPE:
+				length = strlen(curField->name);
+				if (length < 11)
+				{
+					length = 11;
+				}
+				break;
+
 			    case REAL_TYPE:
 				length = strlen(curField->name);
 				if (length < 12)
@@ -252,6 +299,7 @@ handleQuery(sock, q)
 				break;
 
 			    case CHAR_TYPE:
+			    case TEXT_TYPE:
 				length = max(strlen(curField->name),
 						curField->length);
 				break;
@@ -279,7 +327,7 @@ handleQuery(sock, q)
 
 
 
-editQuery(q)
+void editQuery(q)
 	char	*q;
 {
 	char	*filename,
@@ -288,6 +336,14 @@ editQuery(q)
 	int	fd;
 
 	filename = (char *)msql_tmpnam(NULL);
+
+#ifdef OS2
+        if( NULL != strrchr(filename, '\\') )
+                filename = strrchr( filename, '\\' );
+        if( NULL != strrchr(filename, '/') )
+                filename = strrchr( filename, '/' );
+#endif
+
 	fd = open(filename,O_CREAT | O_WRONLY, 0777);
 	editor = (char *)getenv("VISUAL");
 	if (!editor)
@@ -296,32 +352,48 @@ editQuery(q)
 	}
 	if (!editor)
 	{
-#ifdef _PATH_VI
-		editor = _PATH_VI;
+#ifdef OS2
+		editor = "e";
 #else
 		editor = "vi";
 #endif
 	}
 	write(fd,q,strlen(q));
 	close(fd);
-	sprintf(combuf,"%s %s",editor,filename);
+	snprintf(combuf,sizeof(combuf),"%s %s",editor,filename);
 	system(combuf);
 	fd = open(filename,O_RDONLY, 0777);
 	bzero(q,MAX_LEN);
 	read(fd,q,MAX_LEN);
 	close(fd);
 	unlink(filename);
+
+#ifdef OS2
+        /*
+        ** filter all EOFs (EOF --> SPACE)
+        */
+        editor = q;
+        fd     = MAX_LEN;
+        while( 0 < fd )
+        {
+                if( 0x1A == *editor )
+                        *editor = ' ';
+                editor++;
+                fd--;
+        }
+#endif
 }
 	
 
 
-main(argc,argv)
+int main(argc,argv)
 	int	argc;
 	char	*argv[];
 {
 	char	qbuf[MAX_LEN],
 		*cp,
-		*host = NULL;
+		*host = NULL,
+		*confFile = NULL;
 	int	newQ = 1,
 		prompt = 1,
 		sock,
@@ -331,10 +403,16 @@ main(argc,argv)
 		errFlag = 0,
 		qLen = 0;
 	register u_int inchar;
+
+#ifdef WIN32
+	WORD	wVersion;
+	WSADATA	wsaData;
+#endif
+
 	extern	int optind;
 	extern	char *optarg;
 
-        while((c=getopt(argc,argv,"h:"))!= -1)
+        while((c=getopt(argc,argv,"f:h:"))!= -1)
         {
                 switch(c)
                 {
@@ -344,6 +422,14 @@ main(argc,argv)
                                 else
                                         host = optarg;
                                 break;
+
+			case 'f':
+				if (confFile)
+					errFlag++;
+				else
+					confFile = optarg;
+				break;
+
                         case '?':
                                 errFlag++;
                                 break;
@@ -354,29 +440,47 @@ main(argc,argv)
 
 
 	printf("\n");
-	if (argsLeft != 1)
+	if (argsLeft != 1  ||  errFlag)
 	{
 		usage();
 		exit(1);
 	}
 
-	/*
-	** If we don't have a hostname have a look at MSQL_HOST
-	*/
-	if (!host)
+#ifdef WIN32
+	wVersion = MAKEWORD(1,1);
+	if (WSAStartup(wVersion, &wsaData) != 0)
 	{
-		host = (char *)getenv("MSQL_HOST");
+		printf("Can't initialise WinSOCK!\n\n");
+		exit(1);
+	}
+#endif
+
+
+	/*
+	** If we have a config file override the default config
+	*/
+	if (confFile)
+	{
+		msqlLoadConfigFile(confFile);
 	}
 
 	if ((sock = msqlConnect(host)) < 0)
 	{
+#ifdef OS2
+		printf("ERROR : %s\n",msqlGetErrMsg(msqlErrMsg));
+#else
 		printf("ERROR : %s\n",msqlErrMsg);
+#endif
 		exit(1);
 	}
 
 	if (msqlSelectDB(sock, argv[optind]) < 0)
 	{
+#ifdef OS2
+		printf("ERROR : %s\n",msqlGetErrMsg(msqlErrMsg));
+#else
 		printf("ERROR : %s\n",msqlErrMsg);
+#endif
 		exit(1);
 	}
 
@@ -390,6 +494,7 @@ main(argc,argv)
 	(void)bzero(qbuf,sizeof(qbuf));
 	cp = qbuf;
 	printf("\nmSQL > ");
+	fflush(stdout);
 	while(!feof(stdin))
 	{
 		inchar = fgetc(stdin);
