@@ -3,17 +3,13 @@
 **
 **
 ** Copyright (c) 1993-95  David J. Hughes
-** Copyright (c) 1995  Hughes Technologies
+** Copyright (c) 1996-97  Hughes Technologies Pty Ltd.
 **
 ** Permission to use, copy, and distribute for non-commercial purposes,
 ** is hereby granted without fee, providing that the above copyright
 ** notice appear in all copies and that both the copyright notice and this
 ** permission notice appear in supporting documentation.
 **
-** This software is provided "as is" without any expressed or implied warranty.
-**
-** ID = "$Id:"
-**
 */
 
 /*
@@ -38,22 +34,31 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <string.h>
 
+#ifdef WIN32
+#  include <winsock.h>
+#endif
+
+#ifndef OS2
+#  include <unistd.h>
+#  include <stdlib.h>
+#endif
 
 #include <common/portability.h>
 
+#define	_MSQL_SERVER_SOURCE
 #include "msql_priv.h"
-#include "y.tab.h"
+
+#if defined(WIN32)
+#  include "msql_yacc.h"
+#else
+#  include "y.tab.h"
+#endif
 
 #define	REG		register
 #define NUM_HASH	16
 
-#ifndef DEBUG
-#  define malloc(s) 	Malloc(s,__FILE__,__LINE__)
-#  define free(a) 	Free(a,__FILE__,__LINE__)
-#endif
-
-
 /*
 ** Macros for handling the scanner's internal pointers
 */
@@ -61,7 +66,7 @@
 #define yyUnget()	tokPtr--; yytoklen--
 #define yySkip()	(*tokPtr++); tokStart++
 #define yyRevert()	{tokPtr=tokStart; yytoklen=0;}
-#define yyReturn(t)	{tokStart=tokPtr; return(t);}
+#define yyReturn(t)	{tokStart=tokPtr; yyrep=tokenDup(yytext,yytoklen);return(t);}
 
 
 /*
@@ -83,7 +88,8 @@
 ** Debugging macros.
 */
 
-/* #define DEBUG_STATE	/* Define this to watch the state transitions */
+/* Define this to watch the state transitions */
+/* #define DEBUG_STATE	*/
 
 #ifdef DEBUG
 #  define token(x)	(int) "x"
@@ -98,7 +104,9 @@
 #  define CASE(x)	case x:
 #endif
 
-u_char	*yytext 	= NULL;
+u_char	*yytext 	= NULL,
+	*yyprev		= NULL,
+	*yyrep		= NULL;
 u_int	yytoklen	= 0;
 int	yylineno 	= 1;
 static	u_char 		*tokPtr,
@@ -120,10 +128,11 @@ typedef struct symtab_s {
 } symtab_t;
 
 
-static symtab_t symtab[16][16] = {
+symtab_t symtab[16][16] = {
 	{ /* 0 */
-		{ "select",	token(SELECT)},
+		{ "select",	token(MSQL_SELECT)},
 		{ "values",	token(VALUES)},
+		{ "uint",	token(MSQL_UINT)},
 		{ 0,		0}
 	},
 	{ /* 1 */
@@ -137,41 +146,54 @@ static symtab_t symtab[16][16] = {
 	},
 	{ /* 3 */
 		{ "and",	token(AND)},
-		{ "delete",	token(DELETE)},
-		{ "update",	token(UPDATE)},
+		{ "delete",	token(MSQL_DELETE)},
+		{ "update",	token(MSQL_UPDATE)},
+		{ "avl",	token(AVL_INDEX)},
 		{ 0,		0}
 	},
 	{ /* 4 */
 		{ "from",	token(FROM)},
-		{ "create",	token(CREATE)},
+		{ "create",	token(MSQL_CREATE)},
 		{ "primary",	token(PRIMARY)},
-		{ "smallint",	token(INT)},
-		{ "real",	token(REAL)},
+		{ "smallint",	token(MSQL_INT)},
+		{ "real",	token(MSQL_REAL)},
 		{ "as",		token(AS)},
+		{ "min",	token(SET_FUNCT)},
 		{ 0,		0}
 	},
 	{ /* 5 */
-		{ "drop",	token(DROP)},
-		{ "insert",	token(INSERT)},
+		{ "drop",	token(MSQL_DROP)},
+		{ "insert",	token(MSQL_INSERT)},
 		{ "like",	token(LIKE)},
+		{ "text",	token(MSQL_TEXT)},
+		{ "sum",	token(SET_FUNCT)},
 		{ 0,		0}
 	},
 	{ /* 6 */
 		{ "NULL",	token(NULLSYM)},
+		{ "max",	token(SET_FUNCT)},
 		{ 0,		0}
 	},
 	{ /* 7 */
 		{ "asc",	token(ASC)},
+		{ "unique",	token(UNIQUE)},
+		{ "rlike",	token(RLIKE)},
 		{ 0,		0}
 	},
 	{ /* 8 */
 		{ "table",	token(TABLE)},
+		{ "index",	token(INDEX)},
+		{ "clike",	token(CLIKE)},
+		{ "money",	token(MSQL_MONEY)},
+		{ "slike",	token(SLIKE)},
 		{ 0,		0}
 	},
 	{ /* 9 */
 		{ "<=",		token(LE)},
 		{ "all",	token(ALL)},
 		{ "key",	token(KEY)},
+		{ "count",	token(SET_FUNCT)},
+		{ "sequence",	token(SEQUENCE)},
 		{ 0,		0}
 	},
 	{ /* 10 */
@@ -184,29 +206,34 @@ static symtab_t symtab[16][16] = {
 		{ ">=",		token(GE)},
 		{ "by",		token(BY)},
 		{ "null",	token(NULLSYM)},
-		{ "int",	token(INT)},
+		{ "int",	token(MSQL_INT)},
 		{ 0,		0}
 	},
 	{ /* 12 */
 		{ "<",		token(LT)},
 		{ "order",	token(ORDER)},
 		{ "set",	token(SET)},
+		{ "step",	token(STEP)},
 		{ 0,		0}
 	},
 	{ /* 13 */
 		{ "=",          token(EQ)},
+		{ "on",        	token(ON)},
+		{ "value",      token(VALUE)},
 		{ 0,		0}
 	},
 	{ /* 14 */
 		{ ">",          token(GT)},
-		{ "integer",	token(INT)},
-		{ "char",	token(CHAR)},
-		{ "ilike",	token(ILIKE)},
+		{ "integer",	token(MSQL_INT)},
+		{ "char",	token(MSQL_CHAR)},
+		{ "avg",	token(SET_FUNCT)},
+		{ "date",	token(MSQL_DATE)},
 		{ 0,		0}
 	},
 	{ /* 15 */
 		{ "desc",	token(DESC)},
 		{ "limit",	token(LIMIT)},
+		{ "time",	token(MSQL_TIME)},
 		{ 0,		0}
 	}
 };
@@ -215,7 +242,7 @@ static symtab_t symtab[16][16] = {
 
 
 
-msqlInitScanner(buf)
+void msqlInitScanner(buf)
 	u_char	*buf;
 {
 	tokStart = buf;
@@ -257,9 +284,6 @@ int findKeyword(tok,len)
 				found = 0;
 				break;
 			}
-/*
-			if (tolower(*cp2++) != *cp1++)
-*/
 			tmp = *cp2++;
 			if (tmp >64 && tmp<91)
 				tmp+=32;
@@ -276,7 +300,7 @@ int findKeyword(tok,len)
 		if (found)
 		{
 			yytext = (u_char *)stab->name;
-			yylval = (YYSTYPE)stab->tok;
+			yylval = (YYSTYPE)stab->name;
 			return(stab->tok);
 		}
 		stab++;
@@ -333,6 +357,7 @@ int yylex()
 	REG	u_char	t;
 	int	tokval;
 	static	u_char dummyBuf[2];
+	static	char EOI[] = "*end of input*";
 
 
 	/*
@@ -341,8 +366,12 @@ int yylex()
 	** allows the parser to do the right thing with trailing garbage
 	** in the expression.
 	*/
+	if (yyprev)
+		free(yyprev);
+	yyprev = yyrep;
 	if (state == 1000)
 	{
+		yyprev = NULL;
 		return(0);
 	}
 	state = 0;
@@ -371,6 +400,11 @@ int yylex()
 					state = 12;
 					break;
 				}
+				if (c == '_')
+				{
+					state = 18;
+					break;
+				}
 				if (isalpha(c))
 				{
 					state = 1;
@@ -393,15 +427,11 @@ int yylex()
 					else
 						yyUnget();
 				}
-				if ((c == '+') || (c == '-')
-				 || (c == '*') || (c == '/')) {
-					yytext = tokenDup(tokStart,yytoklen);
-					yylval = (YYSTYPE) yytext;
-					if (c == '+') yyReturn(OPPLUS);
-					if (c == '-') yyReturn(OPMINUS);
-					if (c == '*') yyReturn(OPMULT);
-					if (c == '/') yyReturn(OPDIV);
-					}
+				if (c == '-' || c == '+')
+				{
+					state = 9;
+					break;
+				}
 				if (iscompop(c))
 				{
 					state = 10;
@@ -586,7 +616,7 @@ int yylex()
 
 			/* State 13: Complete text string */
 			CASE(13)
-				yyReturn(token(TEXT));
+				yyReturn(token(MSQL_TEXT));
 				break;
 
 
@@ -642,6 +672,24 @@ int yylex()
 			/* Analogy End */
 
 
+			/* State 18 : Incomplete System Variable */
+			CASE(18)
+				c = yyGet();
+				if (isalnum(c) || c == '_')
+				{
+					state = 18;
+					break;
+				}
+				state = 19;
+				break;
+
+
+			/* State 19: Complete Sys Var */
+			CASE(19)
+				yyUnget();
+				yytext = tokenDup(tokStart,yytoklen);
+				yylval = (YYSTYPE) yytext;
+				yyReturn(token(SYS_VAR));
 				
 
 			/* State 999 : Unknown token.  Revert to single char */
@@ -657,6 +705,8 @@ int yylex()
 
 			/* State 1000 : End Of Input */
 			CASE(1000)
+				yytext = EOI;
+				yylval = (YYSTYPE) EOI;
 				yyReturn(token(END_OF_INPUT));
 
 		}
@@ -666,10 +716,30 @@ int yylex()
 
 #ifdef DEBUG
 
+#undef malloc
+#undef free
+
+char *Malloc(size,file,line)
+	int	size;
+	char	*file;
+	int	line;
+{
+	return((char *)malloc(size));
+}
+
+
+void Free(ptr,file,line)
+	char	*ptr;
+	char	*file;
+	int	line;
+{
+	free(ptr);
+}
+
 main()
 {
 	char	*p,
-		tmpBuf[4 * 1024];
+		tmpBuf[10 * 1024];
 
 	(void)bzero(tmpBuf,sizeof(tmpBuf));
 	read(fileno(stdin),tmpBuf,sizeof(tmpBuf));
