@@ -20,12 +20,7 @@
 #define MSQL_PRIV_H
 
 #include "version.h"
-
-#ifdef	BERK_DB
-#  include <db/db.h>
-#else
-#  include "avl_tree.h"
-#endif
+#include "avl_tree.h"
 
 
 /***************************************************************************
@@ -37,6 +32,7 @@
 #define MAX_FIELDS	75		/* Max fields per query */
 #define NAME_LEN	35		/* Field/table name length */
 #define	PKT_LEN		(32*1024)	/* Max size of client/server packet */
+#define MAX_ERR_MSG	200		/* Max len of error msg */
 #define	CACHE_SIZE	6		/* Size of table cache */
 #define	OFB_SIZE	(128-sizeof(u_int))/* Size of overflow buffers */
 #define NUM_INDEX	5		/* Max indices per table */
@@ -119,11 +115,7 @@ typedef struct mindex_s {
 		length,
 		fieldCount,
 		idxType;
-#ifdef BERK_DB
-	DB	*file;
-#else
 	avltree	*tree;
-#endif
 	struct	mindex_s *next;
 } mindex_t;
 
@@ -250,8 +242,6 @@ typedef struct cinfo_s {
 } cinfo_t;
 
 
-
-
 typedef struct cand_s {
 	int	type;		/* Lookup type - SEQ, IDX_ABS, IDX_RANGE */
 	u_int	nextPos,	/* Seq search next pos */
@@ -264,12 +254,8 @@ typedef struct cand_s {
 		*rangeMax;	/* IDX_RANGE stop value */
 	char	*buf,		/* Index value buffer */
 		idx_name[NAME_LEN + 1];
-#ifdef BERK_DB
-	DB      *file;		/* Btree */
-#else
 	avltree	*tree;
 	avl_cur	cursor;
-#endif
 } cand_t;
 
 
@@ -482,7 +468,7 @@ int 	unixToDate __ANSI_PROTO((time_t));
 int 	unixToTime __ANSI_PROTO((time_t));
 void	printDate __ANSI_PROTO((char*, int, int));
 void	printTime __ANSI_PROTO((char*, int, int));
-void	printMoney __ANSI_PROTO((char*, int));
+void	printMoney __ANSI_PROTO((char*, int, int));
 
 
 /* table.c prototypes */
