@@ -20,6 +20,7 @@
 #include <stdio.h>
 #include <fcntl.h>
 #include <limits.h>
+#include <locale.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/time.h>
@@ -168,7 +169,33 @@ RETSIGTYPE sigTrap(sig)
 
 void mmapSyncer()
 {
+	extern	cache_t tableCache[CACHE_SIZE];
+	cache_t	*cur;
+	int	count;
 
+	for (count = 0; count < CACHE_SIZE; count++)
+	{
+		cur = tableCache + count;
+		if (cur->age == 0)
+		{
+			count++;
+			continue;
+		}
+		cur->remapData = 1;
+		cur->remapOverflow = 1;
+	}
+#ifndef WIN32
+        sync();
+#endif  
+#if !defined(WIN32) && !defined(OS2)
+        signal(SIGALRM, mmapSyncer);
+        alarm(msyncTimer);  
+#endif
+}
+
+
+void old_mmapSyncer()
+{
 
 #ifdef HAVE_MSYNC
 	{
@@ -224,9 +251,9 @@ void sendError(fd,err)
 		printf("Send error called\n");
 #	endif
 	if (err)
-		sprintf(packet,"-1:%s\n",err);
+		snprintf(packet,PKT_LEN,"-1:%s\n",err);
 	else
-		sprintf(packet,"-1:%s\n",errMsg);
+		snprintf(packet,PKT_LEN,"-1:%s\n",errMsg);
 	writePkt(fd);
 }
 
@@ -235,7 +262,7 @@ void sendOK(fd)
 #	ifdef DEBUG
 		printf("Send OK called\n");
 #	endif
-	sprintf(packet,"1:\n");
+	strcpy(packet,"1:\n");
 	writePkt(fd);
 }
 
@@ -376,7 +403,7 @@ RETSIGTYPE puntServer(sig)
 void yyerror(s)
         char	*s;
 {
- 	sprintf(packet,"-1:%s at line %d near \"%s %s\"\n", s, 
+ 	snprintf(packet,PKT_LEN,"-1:%s at line %d near \"%s %s\"\n", s, 
 		yylineno, (char *)yyprev != NULL ?(char *)yyprev:"", 
 		(char *)yytext != NULL ?(char *)yytext:"");
 	writePkt(curSock);
@@ -441,16 +468,17 @@ void sendServerStats(sock)
 	u_int	curTime = time(NULL);
 	char	buf[10];
 
-	sprintf(packet,"Mini SQL Version %s\n",SERVER_VERSION);
-	strcat(packet,"Copyright (c) 1993-94 David J. Hughes\n");
-	strcat(packet,"Copyright (c) 1995-97 Hughes Technologies Pty Ltd.\n");
-	strcat(packet,"All rights reserved.\n\n");
+	snprintf(packet,PKT_LEN, "%s%s\n%s\n%s\n%s\n\n", 
+		"Mini SQL Version ", SERVER_VERSION,
+		"Copyright (c) 1993-94 David J. Hughes",
+		"Copyright (c) 1995-97 Hughes Technologies Pty Ltd.",
+		"All rights reserved.");
 	writePkt(sock);
-	sprintf(packet,"Config file      : %s\n",confFile);
+	snprintf(packet,PKT_LEN,"Config file      : %s\n",confFile);
 	writePkt(sock);
-	sprintf(packet,"Max connections  : %d\n",maxCons);
+	snprintf(packet,PKT_LEN,"Max connections  : %d\n",maxCons);
 	writePkt(sock);
-	sprintf(packet,"Cur connections  : %d\n",numCons);
+	snprintf(packet,PKT_LEN,"Cur connections  : %d\n",numCons);
 	writePkt(sock);
 
 #if !defined(WIN32) && !defined(OS2)
@@ -458,11 +486,11 @@ void sendServerStats(sock)
 	pw = getpwuid(uid);
 	if (pw != NULL)
 	{
-		sprintf(packet,"Running as user  : %s\n", pw->pw_name);
+		snprintf(packet,PKT_LEN,"Running as user  : %s\n", pw->pw_name);
 	}
 	else
 	{
-		sprintf(packet,"Running as user  : UID %d\n", uid);
+		snprintf(packet,PKT_LEN,"Running as user  : UID %d\n", uid);
 	}
 	writePkt(sock);
 #endif
@@ -480,9 +508,10 @@ void sendServerStats(sock)
 		if (conArray[loop].user)
 		{
 			cTime = (int)(curTime - conArray[loop].connectTime)/60;
-			sprintf(buf,"%2dH %2dM",cTime/60, cTime%60);
-			sprintf(packet,
-			    " | %3d | %-10s | %-15s | %-10s | %s | %4d | %6d |\n",
+			snprintf(buf,sizeof(buf),
+				"%2dH %2dM",cTime/60, cTime%60);
+			snprintf(packet,PKT_LEN,
+			 " | %3d | %-10s | %-15s | %-10s | %s | %4d | %6d |\n",
 			    loop, conArray[loop].user, 
 			    conArray[loop].host?conArray[loop].host:"UNIX Sock",
 			    conArray[loop].db ? conArray[loop].db : "No DB",
@@ -558,6 +587,7 @@ void main(argc,argv)
 		*cp,
 		*arg,
 		*arg2,
+		tmpChar,
 		path[MAXPATHLEN];
 	FILE	*pidFile;
         extern  char *optarg;
@@ -583,6 +613,9 @@ void main(argc,argv)
 	printf("Copyright (c) 1995-97 Hughes Technologies Pty Ltd.\n");
 	printf("All rights reserved.\n\n");
 
+	/* Try to set the locale but don't bail if it fails (freeBSD) */
+	setlocale(LC_ALL, "");
+	
 	/*
 	** Handle the command line args
 	*/
@@ -610,7 +643,7 @@ void main(argc,argv)
 
 	if (*confFile == 0)
 	{
-		sprintf(confFile,"%s/msql.conf", INST_DIR);
+		snprintf(confFile,sizeof(confFile),"%s/msql.conf", INST_DIR);
 	}
 	printf("\tLoading configuration from '%s'.\n",confFile);
 	msqlLoadConfigFile(confFile);
@@ -668,7 +701,8 @@ void main(argc,argv)
 	initServer();
 	initBackend();
 	readOnly = msqlGetIntConf("system","read_only");
-	(void)sprintf(path,"%s", (char *)msqlGetCharConf("general","pid_file"));
+	(void)snprintf(path, MAXPATHLEN, "%s", 
+		(char *)msqlGetCharConf("general","pid_file"));
 	pidFile = fopen(path,"w");
 	if (!pidFile)
 	{
@@ -713,7 +747,8 @@ void main(argc,argv)
 	/*
 	** Ensure that the correct user owns the database files
 	*/
-	sprintf(path,"%s/msqldb",(char *)msqlGetCharConf("general","inst_dir"));
+	snprintf(path,MAXPATHLEN, "%s/msqldb",
+		(char *)msqlGetCharConf("general","inst_dir"));
 	if (stat(path,&sbuf) < 0)
 	{
 		printf("\nError!  Can't stat '%s'\n\n",path);
@@ -856,7 +891,7 @@ void main(argc,argv)
 			if (sock == IPsock)
 			{
 				size_t	addrLen;
-				struct	hostent *hp;
+				struct	hostent *hp, *hp2;
 
 				addrLen = sizeof(struct sockaddr);
 				getpeername(newSock, (struct sockaddr *)
@@ -870,6 +905,21 @@ void main(argc,argv)
 						(char *)&conArray[newSock].remote.sin_addr,
 						sizeof(conArray[newSock].remote.sin_addr),
 						AF_INET);
+					if (hp)
+					{
+						hp2=(struct hostent*)
+						    gethostbyname(hp->h_name);
+						if (bcmp(hp->h_addr_list[0],
+							hp2->h_addr_list[0],
+							hp2->h_length)!=0)
+						{
+						    sendError(newSock,
+							BAD_HOST_ERROR);
+						    error = 1;
+						    shutdown(newSock,2);
+						    close(newSock);
+						}
+					}
 				}
 				else
 				{
@@ -878,25 +928,27 @@ void main(argc,argv)
 
 				if (!hp)
 				{
-					if (bcmp(&conArray[newSock].local,
-							&conArray[newSock].remote,addrLen)== 0)
+					struct sockaddr_in *locIP, *remIP;
+
+					locIP = &conArray[newSock].local;
+					remIP = &conArray[newSock].remote;
+
+					if (bcmp(&(locIP->sin_addr),
+						&(remIP->sin_addr),
+						sizeof(u_long))== 0)
 					{
-						conArray[newSock].host = (char *)
+						conArray[newSock].host = 
+							(char *)
 							strdup("localhost");
 					}
 					else
 					{
-						conArray[newSock].host = (char *)
+						conArray[newSock].host = 
+							(char *)
 							strdup("UNKNOWN_HOST");
-						msqlDebug(MOD_GENERAL,"Host = UNKNOWN_HOST\n");
+						msqlDebug(MOD_GENERAL,
+							"Host=UNKNOWN_HOST\n");
 					}
-
-					/*
-					sendError(newSock, BAD_HOST_ERROR);
-					error = 1;
-					shutdown(newSock,2);
-					close(newSock);
-					*/
 				}
 				else
 				{
@@ -925,7 +977,7 @@ void main(argc,argv)
 				opt=1;
 				setsockopt(newSock,SOL_SOCKET,SO_KEEPALIVE,
 					(char *) &opt, sizeof(opt));
-				sprintf(packet,
+				snprintf(packet, PKT_LEN,
 					"0:%d:%s\n",
 					PROTOCOL_VERSION,SERVER_VERSION);
 				writePkt(newSock);
@@ -939,7 +991,7 @@ void main(argc,argv)
                                         	sizeof(struct sockaddr));
                                 	bzero(&conArray[newSock].
                                         	remote,sizeof(struct sockaddr));
-
+					numCons--;
 				}
 				else
 				{
@@ -949,7 +1001,7 @@ void main(argc,argv)
 					safeFree(conArray[newSock].user);
 					conArray[newSock].user = (char *)
 						strdup(uname);
-					sprintf(packet,"-100:\n");
+					strcpy(packet,"-100:\n");
 					writePkt(newSock);
 				}
 			}
@@ -1027,7 +1079,15 @@ void main(argc,argv)
 				arg = (char *)strdup(cp);
 				if (debugSet(MOD_QUERY))
 					fprintf(stderr,"\n");
+				tmpChar = 0;
+				if (strlen(arg) > 4096)
+				{
+					tmpChar = *(arg+4096);
+					*(arg+4096) = 0;
+				}
 				msqlDebug(MOD_QUERY,"Query = %s",arg);
+				if (tmpChar)
+					*(arg+4096) = tmpChar;
 				msqlSetDB(conArray[comSock].db);
 				msqlSetPerms(conArray[comSock].access);
 				msqlParseQuery(arg,comSock);
@@ -1177,7 +1237,7 @@ void main(argc,argv)
 					sendError(comSock,PERM_DENIED_ERROR);
 					break;
 				}
-				(void)sprintf(packet,"-100:\n");
+				(void)strcpy(packet,"-100:\n");
 				msqlServerReloadAcls(comSock);
 				writePkt(comSock);
 				break;
@@ -1188,7 +1248,7 @@ void main(argc,argv)
 					sendError(comSock,PERM_DENIED_ERROR);
 					break;
 				}
-				sprintf(packet,"-100:\n");
+				strcpy(packet,"-100:\n");
 				writePkt(comSock);
 				puntServer(-1);
 				exit(0);
@@ -1196,7 +1256,7 @@ void main(argc,argv)
 
 			    case SERVER_STATS:
 				sendServerStats(comSock);
-				sprintf(packet,"-100:\n");
+				strcpy(packet,"-100:\n");
 				writePkt(comSock);
 				break;
 
