@@ -2498,7 +2498,8 @@ int msqlServerUpdate(table,fields,conds,db)
 		fullFlist[MAX_FIELDS],
 		rowLen,
 		res,
-		count;
+		count,
+		updated;
 	u_int	rowNum;
 	row_t	row;
 	cache_t	*cacheEntry;
@@ -2635,18 +2636,14 @@ int msqlServerUpdate(table,fields,conds,db)
 			if (cacheEntry->indices)
 			{
 				if (updateIndices(cacheEntry,cacheEntry->def,
-					rowNum, &row, fullFlist) < 0)
+					rowNum, &row, fullFlist, &updated,
+					candidate->index) < 0)
 				{
 					freeCandidate(candidate);
 					return(-1);
 				}
-				/*
-				** Have to reset this.  If it's an index based
-				** lookup then the delete may have shuffled a 
-				** a dup up the chain.  A getnext would skip the
-				** correct entry.
-				*/
-				resetCandidate(candidate, MSQL_DELETE);
+				if (updated)
+					resetCandidate(candidate, MSQL_DELETE);
 			}
 		}
 		rowNum = getCandidate(cacheEntry, candidate);
