@@ -44,13 +44,15 @@
 #define EX_CONSCHECK 3
 
 char	usage[] = "\n\n\
-usage: 	%s [-h host] [-f conf] [-v] [-c] database [table]\n\n\
+usage: 	%s [-h host] [-f conf] [-v] [-t] [-c] [-w where_clause] database [table]\n\n\
 	Produce an ASCII dump of a database table or an entire database\n\n";
 
 int	verbose = 0,
-	cFlag = 0;
+	cFlag = 0,
+	tFlag = 0;
 int	sock = -1;
 char	insert_pat[5 * 1024],
+	*whereClause = NULL,
 	colDef[1024];
 
 
@@ -181,7 +183,8 @@ int getTableStructure(table)
 	}
 
 	printf("\n#\n# Table structure for table '%s'\n#\n",table);
-	sprintf(insert_pat, "INSERT INTO %s %%s VALUES (", table);
+	snprintf(insert_pat, sizeof(insert_pat),
+		"INSERT INTO %s %%s VALUES (", table);
 	printf("CREATE TABLE %s (\n", table);
 
 	strcpy(colDef,"(");
@@ -301,7 +304,7 @@ void dumpTable(numFields,table)
 	int	numFields;
 	char	*table;
 {
-	char		query[48],
+	char		query[1024],
 			*tmp;
 	m_result 	*res;
 	m_field 	*field;
@@ -309,10 +312,20 @@ void dumpTable(numFields,table)
 	int		i;
 	int		init = 1;
 
+	if (tFlag)
+		return;
 	if (verbose)
 		fprintf(stderr, "Sending SELECT query...\n");
 	printf("\n#\n# Dumping data for table '%s'\n#\n\n",table);
-	sprintf(query, "SELECT * FROM %s", table);
+	if (whereClause)
+	{
+		snprintf(query, sizeof(query), "SELECT * FROM %s where %s", 
+			table, whereClause);
+	}
+	else
+	{
+		snprintf(query, sizeof(query), "SELECT * FROM %s ", table);
+	}
 	if (msqlQuery(sock, query) == -1)
 		DBerror();
 	if (!(res=msqlStoreResult()))
@@ -414,7 +427,7 @@ int main(argc, argv)
 	/*
 	** Check out the args
 	*/
-	while((c=getopt(argc,argv,"h:f:vc"))!= -1)
+	while((c=getopt(argc,argv,"h:f:w:vct"))!= -1)
 	{
 		switch(c)
 		{
@@ -425,6 +438,13 @@ int main(argc, argv)
 					host = optarg;
 				break;
 
+			case 'w':
+				if (whereClause)
+					errFlag++;
+				else
+					whereClause = optarg;
+				break;
+
 			case 'v':
 				if (verbose)
 					errFlag++;
@@ -446,6 +466,13 @@ int main(argc, argv)
                                         cFlag++;
                                 break;
 
+                        case 't':
+                                if (tFlag)
+                                        errFlag++;
+                                else
+                                        tFlag++;
+                                break;
+
 			case '?':
 				errFlag++;
 		}
