@@ -61,6 +61,7 @@
 #include <common/site.h>
 #include <common/portability.h>
 
+#include "avl_tree.h"
 
 
 #if defined(OS2) || defined(WIN32)
@@ -876,8 +877,8 @@ cache_t *joinTables(table1,table2,conds,db,tables, fields)
         	msqlTrace(TRACE_OUT,"joinTables()");
 		return(NULL);
 	}
-	(void)sprintf(tmpTable->resInfo,"'%s (%s+%s)'",tmpTable->table,
-		table1->table, table2->table);
+	(void)snprintf(tmpTable->resInfo,4 * (NAME_LEN + 1),
+		"'%s (%s+%s)'",tmpTable->table, table1->table, table2->table);
 
 
 	/*
@@ -1127,6 +1128,17 @@ int createSortedTable(entry,order)
 	order_t	*order;
 {
 	int	olist[MAX_FIELDS];
+	u_int	rowNum;
+	char	idxPath[255],
+		dataPath[255],
+		oldPath[255];
+	avltree *tree;
+	avl_nod	*node;
+	row_t	row;
+	cache_t	tmp;
+	sblk_t	sblock;
+	avl_cur	cursor;
+
 
 	msqlTrace(TRACE_IN,"createSortedTable()");
 
@@ -1135,6 +1147,10 @@ int createSortedTable(entry,order)
 		msqlTrace(TRACE_OUT,"createSortedTable()");
 		return(-1);
 	}
+	if (entry->sblk->numRows == 0)
+	{
+		return(0);
+	}
 	if (msqlSetupOrder(entry,olist,order) < 0)
 	{
 		msqlTrace(TRACE_OUT,"createSortedTable()");
@@ -1142,10 +1158,143 @@ int createSortedTable(entry,order)
 	}
 
 
-	if (entry->sblk->numRows > 0)
+#ifdef 3_0_CODE
+	/* NOTE : Experimental mSQL 3.0 code */
+
+	/*
+	** OK, this may look wierd.  Sorting a table using any
+	** sorting algorithm sucks for performance.  The engine
+	** can create a new, indexed table at a rate of over 800
+	** rows per second on a Pentium yet a sort of 1200 row
+	** table will take 20 to 30 seconds.  So, a quick way to
+	** handle it is to create a dummy index containing the
+	** sort field.  The rowID is stored in the index also.  A
+	** left to right parse of the index tree once generated will
+	** give us the sorted rowID's.
+	**
+	** Note, this only works for a single field.  For mult-fields
+	** we still do this for the first order field to reduce the
+	** work of the qSort implementation (should be mainly sorted by
+	** the first effort).
+	*/
+
+	snprintf(idxPath,MAXPATHLEN,"%s/msqldb/.tmp/%s.sort",INST_DIR,
+		entry->table);
+	switch(order->type)
+	{
+		case CHAR_TYPE:
+			avlCreate(idxPath,0600,order->length,AVL_CHAR,AVL_DUP);
+			break;
+
+		case INT_TYPE:
+		case UINT_TYPE:
+		case MONEY_TYPE:
+		case TIME_TYPE:
+		case DATE_TYPE:
+			avlCreate(idxPath,0600,order->length,AVL_INT,AVL_DUP);
+			break;
+
+		case REAL_TYPE:
+			avlCreate(idxPath,0600,order->length,AVL_REAL,AVL_DUP);
+			break;
+
+		default:
+			avlCreate(idxPath,0600,order->length,AVL_BYTE,AVL_DUP);
+			break;
+	}
+	tree = avlOpen(idxPath);
+
+	/* 
+	** Scan the table creating the index on the fly 
+	*/
+	rowNum = 0;
+	while(rowNum < entry->sblk->numRows)
+	{
+		if (rowRead(entry,&row, rowNum) < 0)
+			return(-1);
+		if (row.header->active == 0)
+		{
+			rowNum++;
+			continue;
+		}
+		avlInsert(tree,(char *)row.data + olist[0], (off_t)rowNum);
+		rowNum++;
+	}
+
+	/*
+	** Create the dummy output table 
+	*/
+	snprintf(dataPath,MAXPATHLEN,"%s/msqldb/.tmp/%s.tmp",INST_DIR,
+		entry->table);
+	tmp.dataFD = open(dataPath, O_CREAT|O_RDWR, 0600);
+        bzero(&sblock, sizeof(sblock));
+        sblock.version = DB_VERSION;
+        sblock.numRows = sblock.activeRows = 0;
+        sblock.freeList = NO_POS;
+	write(tmp.dataFD,&sblock,SBLK_SIZE);
+
+	tmp.indices = NULL;
+	tmp.def = entry->def;
+	tmp.overflowFD = -1;
+	tmp.dataMap = NULL;
+	tmp.remapData = 1;
+	tmp.remapOverflow = 0;
+	initTable(&tmp, FULL_REMAP);
+	tmp.rowDataLen = entry->rowDataLen;
+	tmp.rowLen = entry->rowDataLen + (8 -
+                ((entry->rowDataLen + HEADER_SIZE) % 8));
+        tmp.row.buf = (u_char *)malloc(entry->rowLen + HEADER_SIZE + 2);
+        tmp.row.header = (hdr_t *)tmp.row.buf;
+        tmp.row.data = tmp.row.buf + HEADER_SIZE;
+        tmp.sblk = (sblk_t *)tmp.dataMap;
+
+
+	/*
+	** Parse the index creating a new data table
+	*/
+	if (order->dir == DESC)
+		node = avlGetLast(tree);
+	else
+		node = avlGetFirst(tree);
+	avlSetCursor(tree, &cursor);
+	while(node)
+	{
+		rowNum = (u_int)node->data;
+		rowRead(entry, &row, rowNum);
+		rowWrite(&tmp, &row, NO_POS);
+		if (order->dir == DESC)
+			node = avlGetPrev(tree, &cursor);
+		else
+			node = avlGetNext(tree, &cursor);
+	}
+
+	/*
+	** Swap the new table into place
+	*/
+	snprintf(oldPath,MAXPATHLEN,"%s/msqldb/.tmp/%s.dat",INST_DIR,
+		entry->table);
+	munmap(entry->dataMap, entry->size);
+	close(entry->dataFD);
+	unlink(oldPath);
+	entry->dataFD = tmp.dataFD;
+	entry->size = tmp.size;
+	entry->dataMap = tmp.dataMap;
+	entry->sblk = (sblk_t *)entry->dataMap;
+	rename(dataPath, oldPath);
+	free(tmp.row.buf);
+
+
+	/*
+	** If it's a Multi-field sort, kick in the qSort
+	*/
+
+	if (order->next)
 	{
 		qSort(entry, order, olist, 0, entry->sblk->numRows - 1);
 	}
+#else  /* 3_0_CODE */
+	qSort(entry, order, olist, 0, entry->sblk->numRows - 1);
+#endif
 
 	msqlTrace(TRACE_OUT,"createSortedTable()");
 	return(0);
@@ -1163,9 +1312,7 @@ int createDistinctTable(entry)
 	u_int	rowNum,
 		curRowNum;
 	int	flist[MAX_FIELDS],
-		res,
-		haveVarChar;
-	field_t	*curField;
+		res;
 	
 
 	if(initTable(entry,FULL_REMAP) < 0)
@@ -1181,15 +1328,6 @@ int createDistinctTable(entry)
 		return(-1);
 	}
 
-	haveVarChar = 0;
-	curField = entry->def;
-	while(curField)
-	{
-		if (curField->type == TEXT_TYPE)
-			haveVarChar++;
-		curField = curField->next;
-	}
-
 	curRowNum = 0;
 	while(rowRead(entry,&row,curRowNum) > 0)
 	{
@@ -1219,14 +1357,14 @@ int createDistinctTable(entry)
 				continue;
 			}
 
-			if (haveVarChar)
-			{
-				res = checkDupRow(entry,cur->data,row.data);
-			}
-			else
-			{
-				res = bcmp(cur->data,row.data,entry->rowLen);
-			}
+			/* RNS
+			 * Originally, bcmp was used on the whole row,
+			 * when there were not TEXT fields, but it does
+			 * not properly handle NULL values.
+			 * So, now, everything is done the long way.
+			 * And, I have removed all the haveVarChar stuff.
+			 */
+			res = checkDupRow(entry,cur->data,row.data);
 			if (res == 0)
 			{
 				deleteRow(entry,rowNum);
@@ -1323,7 +1461,7 @@ int doSelect(cacheEntry, tables, fields,conds,dest,tmpTable)
 
 	if (dest == DEST_CLIENT)
 	{
-		sprintf(packet,"1:%d:\n",numFields);
+		snprintf(packet,PKT_LEN,"1:%d:\n",numFields);
 		writePkt(outSock);
 	}
 
@@ -1337,15 +1475,15 @@ int doSelect(cacheEntry, tables, fields,conds,dest,tmpTable)
 		extractValues(cacheEntry, NULL,fields, flist);
 		formatPacket(packet,fields);
 		writePkt(outSock);
-		sprintf(packet,"-100:\n");
+		strcpy(packet,"-100:\n");
 		writePkt(outSock);
 
 		/*
 		** Send the field info down the line to the client
 		*/
-		sprintf(outBuf,"%d",fields->length);
-		sprintf(outBuf2,"%d",fields->type);
-		sprintf(packet,"%d:%s%d:%s%d:%s%d:%s1:%s1:%s", 
+		snprintf(outBuf,sizeof(outBuf),"%d",fields->length);
+		snprintf(outBuf2,sizeof(outBuf2),"%d",fields->type);
+		snprintf(packet,PKT_LEN,"%d:%s%d:%s%d:%s%d:%s1:%s1:%s", 
 			(int)strlen(fields->table), fields->table,
 			(int)strlen(fields->name), fields->name, 
 			(int)strlen(outBuf2), outBuf2,
@@ -1353,7 +1491,7 @@ int doSelect(cacheEntry, tables, fields,conds,dest,tmpTable)
 			fields->flags & NOT_NULL_FLAG ? "Y":"N",
 			" ");
 		writePkt(outSock);
-		sprintf(packet,"-100:\n");
+		strcpy(packet,"-100:\n");
 		writePkt(outSock);
 		msqlTrace(TRACE_OUT,"doSelect()");
 		freeCandidate(candidate);
@@ -1407,7 +1545,7 @@ int doSelect(cacheEntry, tables, fields,conds,dest,tmpTable)
 	}
 	if (dest == DEST_CLIENT)
 	{
-		sprintf(packet,"-100:\n");
+		strcpy(packet,"-100:\n");
 		writePkt(outSock);
 
 		/*
@@ -1416,9 +1554,9 @@ int doSelect(cacheEntry, tables, fields,conds,dest,tmpTable)
 		curField = fields;
 		while(curField)
 		{
-			sprintf(outBuf,"%d",curField->length);
-			sprintf(outBuf2,"%d",curField->type);
-			sprintf(packet,"%d:%s%d:%s%d:%s%d:%s1:%s1:%s", 
+			snprintf(outBuf,sizeof(outBuf),"%d",curField->length);
+			snprintf(outBuf2,sizeof(outBuf2),"%d",curField->type);
+			snprintf(packet,PKT_LEN,"%d:%s%d:%s%d:%s%d:%s1:%s1:%s", 
 				(int)strlen(curField->table), curField->table,
 				(int)strlen(curField->name), curField->name, 
 				(int)strlen(outBuf2), outBuf2,
@@ -1428,7 +1566,7 @@ int doSelect(cacheEntry, tables, fields,conds,dest,tmpTable)
 			writePkt(outSock);
 			curField = curField->next;
 		}
-		sprintf(packet,"-100:\n");
+		strcpy(packet,"-100:\n");
 		writePkt(outSock);
 	}
 	msqlTrace(TRACE_OUT,"doSelect()");
@@ -1472,7 +1610,8 @@ static int checkConds(conds, tables)
 		}
 		if (!curTable)
 		{
-			sprintf(errMsg,UNSELECT_ERROR,(char *)curCond->table);
+			snprintf(errMsg,MAX_ERR_MSG, UNSELECT_ERROR,
+				(char *)curCond->table);
 			return(-1);
 		}
 		curCond = curCond->next;
@@ -1549,7 +1688,8 @@ int msqlServerSelect(tables,fields,conds,order,db)
 		}
 		if (!curTable)
 		{
-			sprintf(errMsg,UNSELECT_ERROR,curField->table);
+			snprintf(errMsg,MAX_ERR_MSG, UNSELECT_ERROR,
+				curField->table);
 			msqlTrace(TRACE_OUT,"msqlSelect()");
 			return(-1);
 		}
@@ -1569,8 +1709,8 @@ int msqlServerSelect(tables,fields,conds,order,db)
 		{
 			if (join)
 			{
-				sprintf(errMsg, UNQUAL_JOIN_ERROR, 
-					curField->name);
+				snprintf(errMsg, MAX_ERR_MSG, 
+					UNQUAL_JOIN_ERROR, curField->name);
 				msqlTrace(TRACE_OUT,"msqlSelect()");
 				return(-1);
 			}
@@ -1596,7 +1736,8 @@ int msqlServerSelect(tables,fields,conds,order,db)
 		}
 		if (!foundTable)
 		{
-			sprintf(errMsg,UNSELECT_ERROR, curField->table);
+			snprintf(errMsg,MAX_ERR_MSG,UNSELECT_ERROR, 
+				curField->table);
 			msqlTrace(TRACE_OUT,"msqlSelect()");
 			return(-1);
 		}
@@ -1754,8 +1895,8 @@ int msqlServerSelect(tables,fields,conds,order,db)
 			freeTmpTable(cacheEntry);
 		return(-1);
 	}
-	(void)sprintf(tmpTable->resInfo,"'%s (stripped %s)'",
-		tmpTable->table,cacheEntry->table);
+	(void)snprintf(tmpTable->resInfo, 4 * (NAME_LEN + 1),
+		"'%s (stripped %s)'", tmpTable->table,cacheEntry->table);
 	if (doSelect(cacheEntry,tables,fields,conds,
 		DEST_TABLE,tmpTable) < 0)
 	{
