@@ -110,7 +110,8 @@ int openTable(table,db)
 {
 	char	path[MAXPATHLEN];
 
-	(void)sprintf(path,"%s/msqldb/%s/%s.dat",msqlHomeDir,db,table);
+	(void)snprintf(path, MAXPATHLEN, "%s/msqldb/%s/%s.dat",msqlHomeDir,db,
+		table);
 #ifdef OS2
         return(_sopen(path,O_RDWR | O_BINARY, SH_DENYNO, S_IREAD | S_IWRITE));
 #else
@@ -125,7 +126,8 @@ int openOverflow(table,db)
 {
 	char	path[MAXPATHLEN];
 
-	(void)sprintf(path,"%s/msqldb/%s/%s.ofl",msqlHomeDir,db,table);
+	(void)snprintf(path, MAXPATHLEN,"%s/msqldb/%s/%s.ofl",msqlHomeDir,db,
+		table);
 #ifdef OS2
         return(_sopen(path,O_RDWR | O_BINARY, SH_DENYNO, S_IREAD | S_IWRITE));
 #else
@@ -310,7 +312,8 @@ static field_t *readTableDef(table,alias,db)
 	static	char buf[MAX_FIELDS * sizeof(field_t)];
 
 	msqlTrace(TRACE_IN,"readTableDef()");
-	(void)sprintf(path,"%s/msqldb/%s/%s.def",msqlHomeDir,db,table);
+	(void)snprintf(path, MAXPATHLEN, "%s/msqldb/%s/%s.def",msqlHomeDir,
+		db,table);
 #ifdef OS2
         fd = _sopen(path, O_RDONLY | O_BINARY, SH_DENYNO, S_IREAD | S_IWRITE);
 #else
@@ -318,7 +321,7 @@ static field_t *readTableDef(table,alias,db)
 #endif
 	if (fd < 0)
 	{
-		sprintf(errMsg,BAD_TABLE_ERROR,table);
+		snprintf(errMsg, MAX_ERR_MSG, BAD_TABLE_ERROR,table);
 		msqlDebug(MOD_ERR,"Unknown table \"%s\"\n",table);
 		msqlTrace(TRACE_OUT,"readTableDef()");
 		return(NULL);
@@ -326,7 +329,8 @@ static field_t *readTableDef(table,alias,db)
 	numBytes = read(fd,buf,sizeof(buf));
 	if (numBytes < 1)
 	{
-		sprintf(errMsg,TABLE_READ_ERROR,table,(char*)strerror(errno));
+		snprintf(errMsg, MAX_ERR_MSG, TABLE_READ_ERROR,table,
+			(char*)strerror(errno));
 		msqlDebug(MOD_ERR,
 			"Error reading table \"%s\" definition\n",
 			table);
@@ -377,7 +381,7 @@ void cleanTmpDir()
 	DIR	*dirp;
 	char	path[MAXPATHLEN];
 
-	(void)sprintf(path,"%s/msqldb/.tmp",msqlHomeDir);
+	(void)snprintf(path,MAXPATHLEN,"%s/msqldb/.tmp",msqlHomeDir);
 	dirp = opendir(path);
 	if (!dirp)
 	{
@@ -397,7 +401,7 @@ void cleanTmpDir()
 			cur = readdir(dirp);
 			continue;
 		}
-		(void)sprintf(path,"%s/msqldb/.tmp/%s",msqlHomeDir,
+		(void)snprintf(path,MAXPATHLEN,"%s/msqldb/.tmp/%s",msqlHomeDir,
 			cur->d_name);
 		unlink(path);
 		cur = readdir(dirp);
@@ -446,7 +450,8 @@ cache_t *createTmpTable(table1,table2,fields, flag)
 	{
 		tmpfile = cp+1;
 	}
-	(void)sprintf(path,"%s/msqldb/.tmp/%s.dat",msqlHomeDir,tmpfile);
+	(void)snprintf(path,MAXPATHLEN,"%s/msqldb/.tmp/%s.dat",msqlHomeDir,
+		tmpfile);
 
 
 	/*
@@ -455,7 +460,7 @@ cache_t *createTmpTable(table1,table2,fields, flag)
 	new = (cache_t *)malloc(sizeof(cache_t));
 	if (!new)
 	{
-		sprintf(errMsg,TMP_MEM_ERROR);
+		snprintf(errMsg,MAX_ERR_MSG,TMP_MEM_ERROR);
 		msqlDebug(MOD_ERR,"Out of memory for temporary table (%s)\n"
 		,path);
 		msqlTrace(TRACE_OUT,"createTmpTable()");
@@ -471,7 +476,8 @@ cache_t *createTmpTable(table1,table2,fields, flag)
 #endif
 	if (fd < 0)
 	{
-		sprintf(errMsg,TMP_CREATE_ERROR, (char*)strerror(errno));
+		snprintf(errMsg,MAX_ERR_MSG,TMP_CREATE_ERROR, 
+			(char*)strerror(errno));
 		msqlDebug(MOD_ERR,"Couldn't create temporary table (%s)\n",
 			path);
 		(void)free(new);
@@ -484,7 +490,8 @@ cache_t *createTmpTable(table1,table2,fields, flag)
 	sblk.freeList = NO_POS;
 	if (write(fd, &sblk, SBLK_SIZE) < SBLK_SIZE)
 	{
-		sprintf(errMsg,TMP_CREATE_ERROR, (char*)strerror(errno));
+		snprintf(errMsg,MAX_ERR_MSG, TMP_CREATE_ERROR, 
+			(char*)strerror(errno));
 		msqlDebug(MOD_ERR,"Couldn't create temporary table (%s)\n",
 			path);
 		(void)free(new);
@@ -507,7 +514,11 @@ cache_t *createTmpTable(table1,table2,fields, flag)
 #endif
 
 	if (new->dataMap == (caddr_t)-1)
+	{
+		free(new);
+		free(tmpStr);
 		return(NULL);
+	}
 	new->sblk=(sblk_t*)new->dataMap;
 
 
@@ -716,7 +727,8 @@ void freeTmpTable(entry)
 	char	path[MAXPATHLEN];
 
         msqlTrace(TRACE_IN,"freeTmpTable()");
-	(void)sprintf(path,"%s/msqldb/.tmp/%s.dat",msqlHomeDir,entry->table);
+	(void)snprintf(path,MAXPATHLEN, "%s/msqldb/.tmp/%s.dat",msqlHomeDir,
+		entry->table);
 	freeTableDef(entry->def);
 	entry->def = NULL;
 	*(entry->db) = 0;
@@ -724,12 +736,12 @@ void freeTmpTable(entry)
 	*(entry->cname) = 0;
 	entry->age = 0;
 	safeFree(entry->row.buf);
-	if (entry->dataMap != (caddr_t) NULL)
+	if (entry->dataMap != (caddr_t) NULL && (entry->size > 0))
 	{
 		munmap(entry->dataMap,entry->size);
-		entry->dataMap = NULL;
-		entry->size = 0;
 	}
+	entry->dataMap = NULL;
+	entry->size = 0;
 	close(entry->dataFD);
 	closeIndices(entry);
 	(void)free(entry);
@@ -860,18 +872,20 @@ cache_t *loadTableDef(table,cname,db)
 	{
 		msqlDebug(MOD_CACHE,"Removing cache entry %d (%s:%s)\n", 
 			cacheIndex, entry->db, entry->table);
-		if (entry->dataMap != (caddr_t) NULL)
+		if (entry->dataMap != (caddr_t) NULL && entry->size >
+0)
 		{
 			munmap(entry->dataMap,entry->size);
-			entry->dataMap = NULL;
-			entry->size = 0;
 		}
-		if (entry->overflowMap != (caddr_t) NULL)
+		entry->dataMap = NULL;
+		entry->size = 0;
+		if (entry->overflowMap != (caddr_t) NULL  &&
+		    entry->overflowSize > 0)
 		{
 			munmap(entry->overflowMap,entry->overflowSize);
-			entry->overflowMap = NULL;
-			entry->overflowSize = 0;
 		}
+		entry->overflowMap = NULL;
+		entry->overflowSize = 0;
 		(void)close(entry->dataFD);
 		(void)close(entry->overflowFD);
 		closeIndices(entry);
@@ -898,7 +912,8 @@ cache_t *loadTableDef(table,cname,db)
 	}
 	if (!def)
 	{
-		sprintf(errMsg,TABLE_READ_ERROR,table, (char*)strerror(errno));
+		snprintf(errMsg, MAX_ERR_MSG,TABLE_READ_ERROR,table, 
+			(char*)strerror(errno));
 		msqlDebug(MOD_ERR,"Couldn't read table definition for %s\n",table);
 		msqlTrace(TRACE_OUT,"loadTableDef()");
 		return(NULL);
@@ -929,7 +944,7 @@ cache_t *loadTableDef(table,cname,db)
 		*(entry->table)=0;
 		*(entry->db)=0;
 		entry->age=0;
-		sprintf(errMsg,DATA_OPEN_ERROR,tableName, 
+		snprintf(errMsg, MAX_ERR_MSG, DATA_OPEN_ERROR,tableName, 
 			(char *)strerror(errno));
 		msqlTrace(TRACE_OUT,"loadTableDef()");
 		return(NULL);
@@ -943,7 +958,7 @@ cache_t *loadTableDef(table,cname,db)
 		*(entry->table)=0;
 		*(entry->db)=0;
 		entry->age=0;
-		sprintf(errMsg,DATA_OPEN_ERROR,tableName, 
+		snprintf(errMsg,MAX_ERR_MSG,DATA_OPEN_ERROR,tableName, 
 			(char *)strerror(errno));
 		msqlTrace(TRACE_OUT,"loadTableDef()");
 		return(NULL);
@@ -962,7 +977,18 @@ cache_t *loadTableDef(table,cname,db)
 	entry->remapData = 1;
 	entry->overflowMap = NULL;
 	entry->remapOverflow = 1;
-	initTable(entry,FULL_REMAP);
+	if (initTable(entry,FULL_REMAP) < 0)
+	{
+		close(entry->overflowFD);
+		close(entry->dataFD);
+		closeIndices(entry);
+		freeTableDef(entry->def);
+		entry->def = NULL;
+		*(entry->table) = 0;
+		*(entry->db) = 0;
+		entry->age = 0;
+		return(NULL);
+	}
 
 	/*
 	** Set the globals and bail.  We need rowLen + 2 (one for the
@@ -1009,10 +1035,9 @@ int initTable(cacheEntry,mapFlag)
 	{
 		if (cacheEntry->remapData)
 		{
-			if (cacheEntry->dataMap)
+			if (cacheEntry->dataMap && cacheEntry->size)
 			{
-				munmap(cacheEntry->overflowMap,
-					cacheEntry->overflowSize);
+				munmap(cacheEntry->dataMap, cacheEntry->size);
 			}
 			fstat(cacheEntry->dataFD, &sbuf);
 			cacheEntry->size = sbuf.st_size;
@@ -1035,7 +1060,7 @@ int initTable(cacheEntry,mapFlag)
 		}
 		if (cacheEntry->remapOverflow)
 		{
-			if (cacheEntry->overflowMap)
+			if (cacheEntry->overflowMap && cacheEntry->overflowSize)
 			{
 				munmap(cacheEntry->overflowMap,
 					cacheEntry->overflowSize);
@@ -1333,7 +1358,8 @@ int fillRow(entry,row,fields,flist)
 						curField->value->val.charVal);
 					if (length > curField->length)
 					{
-						sprintf(errMsg,VALUE_SIZE_ERROR,
+						snprintf(errMsg,MAX_ERR_MSG,
+							VALUE_SIZE_ERROR,
 							curField->name);
 						return(-1);
 					}
@@ -1363,7 +1389,9 @@ int fillRow(entry,row,fields,flist)
 					cp += sizeof(u_int);
 					inline_bcopy(
 						curField->value->val.charVal,
-						cp, curField->length);
+						cp,
+						(length > curField->length) ? curField->length : length
+						);
 					break;
 
 				case REAL_TYPE:
@@ -1441,7 +1469,8 @@ int updateValues(entry,row,fields,flist)
 						curField->value->val.charVal);
                                         if (length > curField->length)
                                         {
-						sprintf(errMsg,VALUE_SIZE_ERROR,
+						snprintf(errMsg,MAX_ERR_MSG,
+							VALUE_SIZE_ERROR,
                                                         curField->name);
                                                return(-1);
                                         }
@@ -1470,7 +1499,9 @@ int updateValues(entry,row,fields,flist)
 					cp += sizeof(u_int);
 					inline_bcopy(
 						curField->value->val.charVal,
-						cp, curField->length);
+						cp,
+						(length > curField->length) ? curField->length : length
+						);
 					break;
 
 				case REAL_TYPE:
@@ -1663,14 +1694,14 @@ int copyFile(from, to)
 	fd1 = open(from, O_RDONLY);
 	if (fd1 < 0)
 	{
-		sprintf(errMsg,"Copy of '%s' failed (%s)",from,
+		snprintf(errMsg,MAX_ERR_MSG,"Copy of '%s' failed (%s)",from,
 			(char*)strerror(errno));
 		return(-1);
 	}
 	fd2 = open(to, O_CREAT | O_RDWR, 0700);
 	if (fd2 < 0)
 	{
-		sprintf(errMsg,"Copy of '%s' failed (%s)",from,
+		snprintf(errMsg,MAX_ERR_MSG,"Copy of '%s' failed (%s)",from,
 			(char*)strerror(errno));
 		return(-1);
 	}
@@ -1679,8 +1710,8 @@ int copyFile(from, to)
 	{
 		if (write(fd2,buf,numBytes) != numBytes)
 		{
-			sprintf(errMsg,"Copy of '%s' failed (%s)",from,
-				(char*)strerror(errno));
+			snprintf(errMsg,MAX_ERR_MSG,"Copy of '%s' failed (%s)",
+				from, (char*)strerror(errno));
 			return(-1);
 		}
 		numBytes = read(fd1,buf,sizeof(buf));
