@@ -64,21 +64,13 @@
 #define REG     register
 extern	char    errMsg[];
 
-/* RNS
- * ISA_NULL_OP -- macro to determine if an operator is one of the NULL
- * family of operators.  This should probably move to msql_priv.h.
- */
-#ifndef ISA_NULL_OP
-#  define ISA_NULL_OP(op) ((op == EQ_OP) || (op == NE_OP))
-#endif
 
-/* RNS
- * ISA_LIKE_OP -- macro to determine if an operator is one of the LIKE
- * family of operators.  This should probably move to msql_priv.h.
- */
-#ifndef ISA_LIKE_OP
-#  define ISA_LIKE_OP(op) ((op >= LIKE_OP) && (op <= NOT_SLIKE_OP))
-#endif
+
+/*
+** Operator class macros
+*/
+#define ISA_NULL_OP(op) ((op == EQ_OP) || (op == NE_OP))
+#define ISA_LIKE_OP(op) ((op >= LIKE_OP) && (op <= NOT_SLIKE_OP))
 
 
 /****************************************************************************
@@ -1755,7 +1747,7 @@ int matchRow(cacheEntry,row,conds)
 			}
 			if (!foundField)
 			{
-				sprintf(errMsg,BAD_FIELD_ERROR,
+				snprintf(errMsg, MAX_ERR_MSG, BAD_FIELD_ERROR,
 					value->val.identVal->seg1,
 					value->val.identVal->seg2);
 				msqlDebug(MOD_ERR,"Unknown field '%s.%s'\n",
@@ -1775,7 +1767,7 @@ int matchRow(cacheEntry,row,conds)
                                 return(-1);
 			if (res == -1)
 			{
-                        	sprintf(errMsg, SYSVAR_ERROR, 
+                        	snprintf(errMsg, MAX_ERR_MSG, SYSVAR_ERROR, 
 					curCond->value->val.identVal->seg2);
                         	msqlDebug(MOD_ERR,SYSVAR_ERROR, 
 					curCond->value->val.identVal->seg2);
@@ -2027,21 +2019,60 @@ int checkDupRow(entry,data1,data2)
 	res = offset = 0;
 	while(curField)
 	{
-		cp1 = data1+offset+1;
-		cp2 = data2+offset+1;
+		/* RNS
+		 * Start by checking for NULL values.
+		 * For purpose of DISTINCT we will consider
+		 * NULLs equal or not to each other.
+		 */
+		cp1 = data1+offset;
+		cp2 = data2+offset;
+		if (*cp1 != *cp2) {
+			/* RNS
+			 * Here, one is NULL and the other is not,
+			 * therefore, we are distinct and are done.
+			 */
+			res = 1;
+			break;
+		}
+		/* RNS
+		 * Here, both are NULL or both have a value.
+		 */
+		if (*cp1 == 0) {
+			/* RNS
+			 * Both are NULL, we want to continue with the
+			 * next field.
+			 */
+			offset += curField->dataLength + 1;
+			curField = curField->next;
+			continue;
+		}
+		/* RNS
+		 * Now that we're done with NULL checking,
+		 * move to actual data area.
+		 */
+		cp1++;
+		cp2++;
 		switch (curField->type)
 		{
-			case CHAR_TYPE:
-			case REAL_TYPE:
 			case INT_TYPE:
 			case UINT_TYPE:
 			case DATE_TYPE:
+			case TIME_TYPE:
+			case MONEY_TYPE:
 				res = bcmp(cp1, cp2, curField->dataLength);
 				break;
 
+			case CHAR_TYPE:
+				res = bcmp(cp1, cp2, curField->length);
+				break;
+
+			case REAL_TYPE:
+				res = bcmp(cp1+1, cp2+1, curField->length);
+				break;
+
 			case TEXT_TYPE:
 				res = compareVarChar(entry,cp1,cp2,
-					curField->dataLength) == 1;
+					curField->length);
 				break;
 		}
 		if (res != 0)
@@ -2202,14 +2233,41 @@ int compareRows(entry,r1,r2,order,olist)
 
 
 
-void printMoney(buf,val)
+void printMoney(buf,bufLen,val)
 	char	*buf;
 	int	val;
 {
-	double	tmp;
+	char 	*cp;
+	int	len;
 
-	tmp = (double)val / 100;
-	sprintf(buf,"%.2f",tmp);
+	/* 
+	** Due to some wierd floating point rounding errors we have to
+	** do this as text (because 894/100 = 8.93999999999)
+	*/
+	snprintf(buf,bufLen,"%d",val);
+	len = strlen(buf);
+	if (len > 2)
+	{
+		cp = buf + len;
+		*(cp + 1) = 0;
+		*cp = *(cp - 1);
+		cp--;
+		*cp = *(cp - 1);
+		cp--;
+		*cp = '.';
+	}
+	else
+	{
+		cp = buf + len+1;
+		*(cp + 1) = 0;
+		*cp = *(cp - 2);
+		cp--;
+		*cp = *(cp - 2);
+		cp--;
+		*cp = '.';
+		cp--;
+		*cp = '0';
+	}
 }
 
 
