@@ -338,7 +338,8 @@ void checkAuth(level)
 		exit(1);
 	}
 
-	sprintf(qBuf,"select passwd from users where uname = '%s'",
+	sprintf(qBuf,
+	    "select passwd from users where uname = '%s' and namespace = 'SuperUser'",
 		username);
 	if (msqlQuery(sock,qBuf) < 0)
 	{
@@ -1138,8 +1139,8 @@ void userManagement(opt)
 		oldUname = (char *)strdup(uname);
 		uname = msqlEscape(uname);
 		sprintf(qBuf,
-		    "select full_name,passwd from users where uname='%s'",
-			uname);
+		    "select full_name,passwd from users where uname='%s' and namespace = '%s'",
+			uname, curNS);
 		if (msqlQuery(sock, qBuf) < 0)
 		{
 			runError(msqlErrMsg);
@@ -1879,7 +1880,7 @@ void areaManagementAction(opt)
 		name = msqlEscape(name);
 		msqlNS = msqlEscape(curNS);
 
-		sprintf(qBuf,"select url from areas where name='%s'",name);
+		sprintf(qBuf,"select url from areas where name='%s' and namespace = '%s'",name, curNS);
 		if (msqlQuery(sock,qBuf) < 0)
 		{
 			runError(msqlErrMsg);
@@ -2004,8 +2005,8 @@ void areaManagementAction(opt)
 		}
 		if (strcmp(oldName,name) != 0)
 		{
-			sprintf(qBuf,"select url from areas where name='%s'",
-				name);
+			sprintf(qBuf,"select url from areas where name='%s' and namespace='%s'",
+				name, curNS);
 			if (msqlQuery(sock,qBuf) < 0)
 			{
 				runError(msqlErrMsg);
@@ -2024,13 +2025,13 @@ void areaManagementAction(opt)
 		/*
 		** Blow away the old stuff
 		*/
-		sprintf(qBuf,"delete from areas where name='%s'",oldName);
+		sprintf(qBuf,"delete from areas where name='%s' and namespace = '%s'",oldName, curNS);
 		if (msqlQuery(sock,qBuf) < 0)
 		{
 			runError(msqlErrMsg);
 			exit(1);
 		}
-		sprintf(qBuf,"delete from area_acl where name='%s'",oldName);
+		sprintf(qBuf,"delete from area_acl where name='%s' and namespace='%s'",oldName, curNS);
 		if (msqlQuery(sock,qBuf) < 0)
 		{
 			runError(msqlErrMsg);
@@ -2126,7 +2127,10 @@ void areaManagement(opt)
 		printf("<INPUT TYPE=HIDDEN NAME=NS VALUE=%s>\n",urlNS);
 		printf("<SELECT SIZE=8 NAME=AREA>\n");
 
-		if(msqlQuery(sock,"select name from areas order by name")<0)
+		sprintf(qBuf,
+		  "select name from areas where namespace='%s' order by name",
+			curNS);
+		if(msqlQuery(sock,qBuf) < 0)
 		{
 			runError(msqlErrMsg);
 			exit(1);
@@ -2187,7 +2191,7 @@ void areaManagement(opt)
 		}
 		area = msqlEscape(area);
 
-		sprintf(qBuf, "select url from areas where name='%s'", area);
+		sprintf(qBuf, "select url from areas where name='%s' and namespace = '%s'", area, curNS);
 		if (msqlQuery(sock, qBuf) < 0)
 		{
 			runError(msqlErrMsg);
@@ -2206,7 +2210,7 @@ void areaManagement(opt)
 		printf("</TABLE><P>\n");
 
 		sprintf(qBuf,
-			"select offset,group,acl from area_acl where name='%s' order by offset", area);
+			"select offset,group,acl from area_acl where name='%s' and namespace='%s' order by offset", area, curNS);
 		if (msqlQuery(sock, qBuf) < 0)
 		{
 			runError(msqlErrMsg);
@@ -2333,7 +2337,7 @@ void areaManagement(opt)
 		oldArea = (char *)strdup(area);
 		area = msqlEscape(area);
 		sprintf(qBuf,
-		    "select url from areas where name='%s'",area);
+		    "select url from areas where name='%s' and namespace='%s'",area, curNS);
 		if (msqlQuery(sock, qBuf) < 0)
 		{
 			runError(msqlErrMsg);
@@ -2366,7 +2370,7 @@ void areaManagement(opt)
 			exit(1);
 		}
 		res = msqlStoreResult();
-		sprintf(qBuf,"select offset,group,acl from area_acl where name='%s' order by offset",area);
+		sprintf(qBuf,"select offset,group,acl from area_acl where name='%s' and namespace = '%s' order by offset",area, curNS);
 		if (msqlQuery(sock, qBuf) < 0)
 		{
 			runError(msqlErrMsg);
@@ -3936,8 +3940,8 @@ void privilegeManagement(opt)
 			runError("Missing username");
 			exit(1);
 		}
-		sprintf(qBuf,"select full_name from users where uname='%s'",
-			uname);
+		sprintf(qBuf,"select full_name from users where uname='%s' and namespace='%s'",
+			uname, curNS);
 		if (msqlQuery(sock, qBuf) < 0)
 		{
 			runError(msqlErrMsg);
